import csv
from tqdm import trange, tqdm
import logging
from typing import Union

logger = logging.getLogger(__name__)

def export_coverages_as_csv(coverage_maps: dict[str, dict[int, int]], length, output_path: str, sparse: bool):
    logger.info("Beginning export of coverage data as CSV")
    if sparse:
        logger.info("CSV will be sparse")
    with open(output_path, "w") as coverage_stream:
        csv_writer = csv.writer(coverage_stream)
        coverage_names = list(coverage_maps.keys())
        headers = ["position", *coverage_names]
        logger.debug("Using the following headers: %s", headers)
        csv_writer.writerow(headers)
        for i in trange(length, unit="row", smoothing=0.5):
            row = [i]
            coverage_sum = 0
            for coverage_name in coverage_names:
                coverage = coverage_maps[coverage_name][i] if i in coverage_maps[coverage_name] else 0
                row.append(coverage)
                coverage_sum += coverage
            if sparse:
                if coverage_sum > 0:
                    csv_writer.writerow(row)
            else:
                csv_writer.writerow(row)

def export_centers_as_csv(stat_centers: dict[str, dict[int, float]], length: int, output_path: str, sparse: bool):
    logger.info("Beginning export of statistical center measures")
    if sparse:
        logger.info("Centers CSV will be sparse.")
    with open(output_path, "w") as centers_stream:
        csv_writer = csv.writer(centers_stream)
        measure_names = list(stat_centers.keys())
        headers = ["position", *measure_names]
        logger.debug("Using the following headers: %s", headers)
        csv_writer.writerow(headers)
        for i in trange(length, unit="row", smoothing=0.5):
            row: list[Union[int, float]] = [i]
            has_data = True
            for measure_name in measure_names:
                if not i in stat_centers[measure_name]:
                    has_data = False
                    break
                row.append(stat_centers[measure_name][i] if i in stat_centers[measure_name] else 0)
            if sparse:
                if has_data:
                    csv_writer.writerow(row)
            else:
                if not has_data:
                    row.extend([0] * len(measure_names))
                csv_writer.writerow(row)

def export_low_coverage_csv(ranges: dict[str, list[list[int]]], length: int, output_path: str):
    logger.info("Beginning export of low coverage areas CSV")
    with open(output_path, "w") as low_coverage_stream:
        csv_writer = csv.writer(low_coverage_stream)
        rows = ["sample", "low coverage ranges"]
        csv_writer.writerow(rows)
        for sample_name, low_cov in tqdm(ranges.items(), unit="sample"):
            pretty_ranges = ", ".join([str(cov_range) for cov_range in low_cov])
            csv_writer.writerow([sample_name, pretty_ranges])